# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: measurements.proto
# Protobuf Python Version: 5.27.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    0,
    '',
    'measurements.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12measurements.proto\"\x85\x04\n\x16MeasurementApiProtobuf\x12/\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x1e.MeasurementApiProtobuf.Column\x12\x12\n\nrows_count\x18\x02 \x01(\r\x12\x14\n\x0c\x63urrent_page\x18\x03 \x01(\r\x12\x15\n\rrows_per_page\x18\x04 \x01(\r\x1aH\n\x06\x43olumn\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\x12.\n\x06values\x18\x02 \x01(\x0b\x32\x1e.MeasurementApiProtobuf.Values\x1a\xc5\x01\n\x06Values\x12\x39\n\x07strings\x18\x01 \x01(\x0b\x32&.MeasurementApiProtobuf.RepeatedStringH\x00\x12\x37\n\x06\x66loats\x18\x02 \x01(\x0b\x32%.MeasurementApiProtobuf.RepeatedFloatH\x00\x12\x37\n\x06ints64\x18\x03 \x01(\x0b\x32%.MeasurementApiProtobuf.RepeatedInt64H\x00\x42\x0e\n\x0coneof_values\x1a\x1f\n\x0eRepeatedString\x12\r\n\x05\x61rray\x18\x01 \x03(\t\x1a\"\n\rRepeatedFloat\x12\x11\n\x05\x61rray\x18\x01 \x03(\x02\x42\x02\x10\x01\x1a\"\n\rRepeatedInt64\x12\x11\n\x05\x61rray\x18\x01 \x03(\x03\x42\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'measurements_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDFLOAT'].fields_by_name['array']._loaded_options = None
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDFLOAT'].fields_by_name['array']._serialized_options = b'\020\001'
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDINT64'].fields_by_name['array']._loaded_options = None
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDINT64'].fields_by_name['array']._serialized_options = b'\020\001'
  _globals['_MEASUREMENTAPIPROTOBUF']._serialized_start=23
  _globals['_MEASUREMENTAPIPROTOBUF']._serialized_end=540
  _globals['_MEASUREMENTAPIPROTOBUF_COLUMN']._serialized_start=163
  _globals['_MEASUREMENTAPIPROTOBUF_COLUMN']._serialized_end=235
  _globals['_MEASUREMENTAPIPROTOBUF_VALUES']._serialized_start=238
  _globals['_MEASUREMENTAPIPROTOBUF_VALUES']._serialized_end=435
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDSTRING']._serialized_start=437
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDSTRING']._serialized_end=468
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDFLOAT']._serialized_start=470
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDFLOAT']._serialized_end=504
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDINT64']._serialized_start=506
  _globals['_MEASUREMENTAPIPROTOBUF_REPEATEDINT64']._serialized_end=540
# @@protoc_insertion_point(module_scope)
